@@ -3,7 +3,7 @@ package HTML::TableContent::Table::Header;
 use Moo;
 use HTML::TableContent::Table::Row::Cell;
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 extends 'HTML::TableContent::Element';
 
@@ -31,6 +31,14 @@ sub get_cell { return $_[0]->cells->[ $_[1] ]; }
 
 sub get_first_cell { return $_[0]->get_cell(0); }
 
+sub get_last_cell { return $_[0]->get_cell( $_[0]->cell_count - 1 ); }
+
+sub unique_cells {
+    my $count_cells = { };
+    map { $count_cells->{$_->text}++ } $_[0]->all_cells;
+    return $count_cells;   
+}
+
 1;
 
 __PACKAGE__->meta->make_immutable;
@@ -43,7 +51,7 @@ HTML::TableContent::Table::Header - base class for table headers.
 
 =head1 VERSION
 
-Version 0.16 
+Version 0.17 
 
 =head1 SYNOPSIS
 
