@@ -3,12 +3,12 @@ package HTML::TableContent::Template;
 use strict;
 use warnings;
 use Carp qw/croak/;
-
+use MooX::ReturnModifiers;
 use HTML::TableContent::Table;
 use HTML::TableContent::Table::Caption;
 use HTML::TableContent::Table::Header;
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 my %TABLE = (
     caption => 'HTML::TableContent::Table::Caption',
@@ -22,14 +22,7 @@ sub import {
     
     my $target = caller;
     
-    for my $needed_method (qw/with around has/) {
-        next if $target->can($needed_method);
-        croak "Can't find method <$needed_method> in <$target>";
-    }
-
-    my $with = $target->can('with');
-    my $around = $target->can('around');
-    my $has = $target->can('has');
+    my %modifiers = return_modifiers($target);
 
     my @target_isa;
 
@@ -65,8 +58,10 @@ sub import {
         }';
     }
 
+    $modifiers{has}->( 'package' => ( is => 'ro', lazy => 1, default => sub { return $target; } ) );
+
     my $apply_modifiers = sub {
-        $with->('HTML::TableContent::Template::Base');
+       $modifiers{with}->('HTML::TableContent::Template::Base');
     };
 
     for my $element (keys %TABLE) {
@@ -78,7 +73,7 @@ sub import {
             my %filtered_attributes = _filter_attributes($name, $element, %attributes);
 
             if ( $element =~ m{caption|header}ixms ){ 
-                $has->( $name => %filtered_attributes );
+                $modifiers{has}->( $name => %filtered_attributes );
             }
 
             delete $filtered_attributes{default};
@@ -87,7 +82,7 @@ sub import {
            
             my $spec = sprintf('_%s_spec', $element);
             if ( $element =~ m{row|cell}ixms ) {
-                $around->(
+                $modifiers{around}->(
                     $spec => sub {
                         my ( $orig, $self ) = ( shift, shift );
                         return $self->$orig(@_), %$element_data;
@@ -95,7 +90,7 @@ sub import {
                 );
             } else {
                 push @element, $element_data;
-                $around->(
+                $modifiers{around}->(
                     $spec => sub {
                         my ( $orig, $self ) = ( shift, shift );
                         return $self->$orig(@_), \@element;
@@ -134,7 +129,7 @@ sub _filter_attributes {
         $attributes{default} = sub { return \%tattr; };
     } else {
         my $class = $TABLE{$element};
-        $attributes{default} = sub { my %dirt = %tattr; return $class->new(\%dirt); };
+        $attributes{default} = sub { my %aah = %tattr; return $class->new(\%aah); };
     }
 
     return %attributes;
@@ -150,7 +145,7 @@ HTML::TableContent::Template - MooX Template Tables.
 
 =head1 VERSION
 
-Version 0.16
+Version 0.17
 
 =cut
 
